;nyquist plug-in
;version 3
;type process
;name "Level Speech..."
;action "Leveling speech dynamics..."

;; LevelSpeech.ny by Steve Daulton (www.easyspacepro.com). March 2014
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

;control ratio "Leveling amount (%)" real "" 50 0 100
;control threshold "Threshold (dB)" real "" -25 -50 0

(setq ratio (min 1 (max 0 (/ ratio 100))))
(setq ratio (power ratio (/ 12.0)))
(setq threshold (db-to-linear (min 0 threshold)))

(defun get-env (sig)
  (let* ((sr (snd-srate sig))
         (risetime 0.5)   ; seconds
         (falltime 1.0)  ; seconds
         (lookahead (truncate (* risetime sr)))
         (env (snd-follow sig threshold risetime falltime lookahead))
         ; prevent boost at end of selection
         (env (sim env (snd-const 1 dur sr 1)))
         (env (extract-abs (+ risetime (/ 20 sr)) (+ risetime (/ 20 sr) dur) env))
         (env (sum (- 1 ratio)(mult ratio env)))
         (env (snd-exp (diff 0 (snd-log env)))))
    env))

;; absolute maximum signal
(defun max-sig (sig)
  (if (arrayp sig)
      (s-max (snd-abs (aref sig 0))(snd-abs (aref sig 1)))
      (snd-abs sig)))

;;; brick wall limiter
(defun limit (s-in)   
  (let* ((la-time (/ 300.0))                      ; lookahead time (seconds)
         (la-s (round (* la-time *sound-srate*))) ; lookahead samples
         (pad-time (* 3 la-time))                 ; padding required at start (s)
         (pad-s (* 3 la-s))                       ; padding smaples
         (padding (snd-const (peak s-in pad-s) 0 *sound-srate* (* 3.0 la-time)))
         (peak-env (snd-avg s-in (* 4 la-s) la-s OP-PEAK))
         (peak-env (sim padding (at-abs pad-time (cue peak-env))))
         (peak-env (extract 0 1 (s-max 1 peak-env))))
    (mult s-in
      (snd-exp 
        (mult -1 (snd-log peak-env))))))

(setq dur (get-duration 1))

(cond
  ((< dur 2) "Error.\nSelection must be at least 2 seconds.")
  (t (let ((s (multichan-expand #'limit s)))
       (mult 0.99 s (get-env (max-sig s))))))
